#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
"""
Tests for solicitor.py
"""
# pylint: disable=E0401, C0301, C0116, R0201, W0621
from unittest.mock import patch

import pytest
from ingest_envoy.solicitor import Solicitor

SOLICIT_WORKFLOWS_PATCH = "ingest_envoy.solicitor.Solicitor.solicit_contents"


@pytest.fixture(scope="function")
def solicitor() -> Solicitor:
    with patch(SOLICIT_WORKFLOWS_PATCH) as mock_content:
        mock_content.return_value = expected_metadata
        return Solicitor("", "calibration", ["http://capability:3457", "http://workflow:3456"])


expected_metadata = {
    "fileSetIds": "16B-069_sb32814386_1_001.57685.66193635417",
    "workflowName": "std_calibration",
    "systemId": "2",
    "creationTime": "2021-07-06T21:50:48",
    "productLocator": "uid://evla/execblock/48ba4c9d-d7c7-4a8f-9803-1115cd52459b",
    "projectMetadata": {
        "projectCode": "16B-069",
        "title": "JVLA monitoring of an in-progress extreme scattering event",
        "telescope": "EVLA",
        "startTime": "57685.661952546296",
        "observer": "Mickey Mouse",
    },
    "destinationDirectory": "/lustre/aoc/cluster/pipeline/docker/workspaces/spool/tmp576dknjp",
}


class TestSolicitor:
    def test_solicit_contents(self):
        with patch(SOLICIT_WORKFLOWS_PATCH) as mock_content:
            mock_content.return_value = expected_metadata
            solicitor = Solicitor("", "calibration", ["http://capability:3457", "http://workflow:3456"])
            mock_content.assert_called()
            assert solicitor.metadata == expected_metadata

    def test_solicit_workflow_directory_name(self, solicitor: Solicitor):
        expected_workflow_dir_name = "tmp576dknjp"
        name = solicitor.solicit_workflow_directory_name()
        assert name == expected_workflow_dir_name

    def test_solicit_processing_creation_time(self, solicitor: Solicitor):
        expected_corrected_time = "2021_07_06T21_50_48"
        time_string = solicitor.solicit_processing_creation_time()
        assert time_string == expected_corrected_time

    def test_solicit_parameters_std_cal(self, solicitor: Solicitor):
        expected_parameters = {
            "sdmId": "16B-069_sb32814386_1_001.57685.66193635417",
            "telescope": "EVLA",
            "workflowName": "std_calibration",
            "workflowRequestId": "2",
            "spl": "uid://evla/execblock/48ba4c9d-d7c7-4a8f-9803-1115cd52459b",
            "processingStart": "2021_07_06T21_50_48",
            "destinationDir": "/somewhere/over/the/rainbow/tmp576dknjp",
            "workflowDir": "tmp576dknjp",
            "initialVersionDir": "tmp24i4o1yl",
            "multiVersion": True,
        }
        with patch(SOLICIT_WORKFLOWS_PATCH):
            with patch("ingest_envoy.solicitor.Solicitor.solicit_initial_directory_name", return_value="tmp24i4o1yl"):
                parameters = solicitor.solicit_parameters()
                assert parameters["workflowDir"] == expected_parameters["workflowDir"]
                assert parameters["initialVersionDir"] == expected_parameters["initialVersionDir"]

    def test_solicit_parameters_image(self, solicitor: Solicitor):
        initial_version_dir = "initial_version_dir"
        cal_source_dir = "tmp576dknjp"
        metadata = {
            "fileSetIds": "brain_000.58099.67095825232",
            "workflowName": "std_cms_imaging",
            "systemId": "12",
            "creationTime": "2021-07-29T14:26:31",
            "productLocator": "uid://evla/execblock/ec082e65-452d-4fec-ad88-f5b4af1f9e36",
            "calProductLocator": "uid://eenie/meanie/miney/moe",
            "projectMetadata": {
                "projectCode": "Operations",
                "title": "",
                "telescope": "EVLA",
                "startTime": "58099.6710792824",
                "observer": "Just Us Chickens",
            },
            "destinationDirectory": "/lustre/aoc/cluster/pipeline/docker/workspaces/spool/" + initial_version_dir,
            "calibrationSourceDirectory": f"/my/stuff/{cal_source_dir}/working",
            "cmsName": "brain_000.58099.67095825232.ms",
        }
        with patch(SOLICIT_WORKFLOWS_PATCH, return_value=metadata):
            with patch(
                "ingest_envoy.solicitor.Solicitor.solicit_initial_directory_name", return_value=initial_version_dir
            ):
                solicitor.argument = "image"
                parameters = solicitor.solicit_parameters()
                assert parameters["workflowDir"] == cal_source_dir
                assert parameters["calSpl"] == metadata["calProductLocator"]
                assert parameters["multiVersion"] is True
